using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Runtime.Serialization;
using System.Windows.Forms;
using System.IO;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Styles;

using Slb.Icp.Forms;
using Slb.Icp.Data.Model;

namespace Slb.Icp.Forms.Grid
{
	/// <summary>
	/// Summary description for UnitTextBoxCell.
	/// </summary>
	public class DoubleUnitTextBoxCellModel : GridGenericControlCellModel
	{
		private static Size unitTextBoxSize = Size.Empty;
		private const int DOUBLEUNIT_HEIGHT = 21;

		public static Size UnitTextBoxSize
		{
			get { return unitTextBoxSize; }
			set { unitTextBoxSize = value;}
		}
		

		public DoubleUnitTextBoxCellModel(GridModel gridModel) : base(gridModel)
		{
			
		}

		protected DoubleUnitTextBoxCellModel(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}

		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new DoubleUnitTextBoxCellRenderer(control , this );
		}


		public static void InitializeUnitTextBox(DoubleUnitTextBox unitBox, GridStyleInfo style)
		{
			DoubleUnitTextBoxStyleProperties styleProperties = new DoubleUnitTextBoxStyleProperties(style);
			unitBox.UnitVisible = styleProperties.UnitVisible;
			unitBox.UnitSymbol =  styleProperties.UnitSymbol;
			unitBox.Measurement = styleProperties.Measurement;
			object val = style.CellValue;
			if ( val is UDouble )
				InitializeUnitTextBox(unitBox , style.CellValue);
			else
				InitializeUnitTextBox(unitBox, styleProperties.DefaultValue);
		}

		public static void InitializeUnitTextBox(DoubleUnitTextBox unitBox, object val )
		{
			if ( val is UDouble )						
				unitBox.Value = (UDouble)val;
		}

		public static void InitializeLabel(Graphics g, Rectangle clientRectangle, GridStyleInfo style, Font font)
		{
			DoubleUnitTextBoxStyleProperties sP = new DoubleUnitTextBoxStyleProperties(style);
			string text;
			Slb.Ocean.Units.Measurement measurement = Slb.Ocean.Units.Measurement.GetMeasurement(sP.Measurement);
			if ( style.CellValue != null && style.CellValue is UDouble)
			{								
				text = ((UDouble) style.CellValue).ToString(measurement, sP.UseUnitSystem );
			}
			else
			{
				if ( sP.DefaultValue != null )
					text = sP.DefaultValue.ToString(measurement, sP.UseUnitSystem );
				else
					text = "";
			}
			GridStaticCellRenderer.DrawText(g,text,font,clientRectangle,style,style.TextColor);

		}


		protected override Size OnQueryPrefferedClientSize(Graphics g, int rowIndex, int colIndex, GridStyleInfo style, GridQueryBounds queryBounds)
		{
			Size size =	base.OnQueryPrefferedClientSize (g, rowIndex, colIndex, style, queryBounds);
			return new Size(size.Width , DOUBLEUNIT_HEIGHT);
		}


	}

	public class DoubleUnitTextBoxCellRenderer : GridGenericControlCellRenderer
	{
		private const int DOUBLEUNIT_HEIGHT = 21;
		DoubleUnitTextBoxCellModel cellModel;
		DoubleUnitTextBox newTextBox = new DoubleUnitTextBox();
		Syncfusion.Windows.Forms.Tools.AutoLabel label = new Syncfusion.Windows.Forms.Tools.AutoLabel();
		
		public DoubleUnitTextBoxCellRenderer(GridControlBase controlBase, GridCellModelBase modelBase) : base ( controlBase , modelBase)
		{
			this.cellModel = (DoubleUnitTextBoxCellModel)modelBase;
			SupportsFocusControl = true;
			this.FixControlParent(label);
			this.SetControl (newTextBox);
		}

		protected override void Dispose(bool disposing)
		{
			if ( disposing )
			{
				this.newTextBox.Dispose();
				this.label.Dispose();
				this.cellModel.Dispose();
			}
			base.Dispose (disposing);
		}

		protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
		{
			try
			{
				clientRectangle.Size = GridUtil.Max
					(new Size(clientRectangle.Width , clientRectangle.Height) , 
					new Size(clientRectangle.Width,DOUBLEUNIT_HEIGHT));
				if ( this.ShouldDrawFocused(rowIndex , colIndex) == true )
				{								
					style.Control = this.newTextBox;
					DoubleUnitTextBoxCellModel.InitializeUnitTextBox(this.newTextBox , style);	
				}
				else
				{	
					DoubleUnitTextBoxCellModel.InitializeLabel(g,clientRectangle,style,style.GdipFont);	
					
				}
				base.OnDraw (g, clientRectangle, rowIndex, colIndex, style);
			}
			catch ( Exception et )
			{
				Console.WriteLine(et.Message);
			}
			

		}
	
		protected override void InitializeControlText(object controlValue)
		{
			DoubleUnitTextBoxCellModel.InitializeUnitTextBox(newTextBox , controlValue);
		}

		protected /*internal*/ override void OnEditingComplete()
		{
	
			object cellValue = this.Grid.Model[RowIndex,ColIndex].CellValue;
			if ((cellValue== null )	||	
				!(cellValue is UDouble) ||
				newTextBox.Value != (UDouble)cellValue)
			{
				this.Grid.Model[RowIndex , ColIndex].CellValue = newTextBox.Value;
			}
		}

	}

	public class DoubleUnitTextBoxStyleProperties : GridStyleInfoCustomProperties	
	{
		static Type type =	typeof(DoubleUnitTextBoxStyleProperties);
		readonly static StyleInfoProperty MeasurementProperty = CreateStyleInfoProperty(type,"Measurement");
		readonly static StyleInfoProperty UnitVisibleProperty = CreateStyleInfoProperty(type,"UnitVisible");
		readonly static StyleInfoProperty UnitSymbolProperty  = CreateStyleInfoProperty(type,"UnitSymbol");
		readonly static StyleInfoProperty UseUnitSystemProperty = CreateStyleInfoProperty(type,"UseUnitSystem");
		readonly static StyleInfoProperty DefaultValueProperty = CreateStyleInfoProperty(type,"DefaultValue");
		
		public DoubleUnitTextBoxStyleProperties(GridStyleInfo style) : base (style)
		{
		}
		
		public DoubleUnitTextBoxStyleProperties():base()
		{
		}


		[
		Description("Gets or sets the Measurement of the DoubleUnitTextBox."),
		Browsable(true),
		Category("DoubleUnitTextBox")
		]
		public string Measurement
		{
			get { return (string)style.GetValue(MeasurementProperty) ; }
			set { style.SetValue(MeasurementProperty, value) ; }
		}

		[
		Description("Gets or sets the UnitVisible Property of the DoubleUnitTextBox."),
		Browsable(true),
		Category("DoubleUnitTextBox")
		]
		public bool UnitVisible
		{
			get { return Convert.ToBoolean(style.GetValue(UnitVisibleProperty));}
			set { style.SetValue(UnitVisibleProperty,value); }
		}
		
		[
		Description("Gets or sets the UnitSymbol of the DoubleUnitTextBox."),
		Browsable(true),
		Category("DoubleUnitTextBox")
		]
		public string UnitSymbol
		{
			get { return (string)style.GetValue(UnitSymbolProperty); }
			set { style.SetValue(UnitSymbolProperty , value); }
		}
		[
		Description("Gets or sets the UseUnitSystem property used in udouble string conversion."),
		Browsable(true),
		Category("DoubleUnitTextBox")
		]
		public bool UseUnitSystem
		{
			get { return Convert.ToBoolean(style.GetValue(UseUnitSystemProperty)); }
			set { style.SetValue(UseUnitSystemProperty,value); }
		}
		public UDouble DefaultValue
		{
			get 
			{
				object val = style.GetValue(DefaultValueProperty);
				if ( val is UDouble )
					return (UDouble)val;
				return null;
			}
			set 
			{
				style.SetValue(DefaultValueProperty , value );
			}
		}

	}

}

